
; *****   abc-26.asm

; ***** / One Button Clock with Touch Plate Interface

;Modified: June 8, 2003


.include "2313def.inc"


; R0-R2 Idelay and delay routines (Idelay switch interruptible)
; R5-R6 is clock counter low, mid, and high
; R8-R10 Time low, high, am/pm
; R11-R13 Scratch
; R14 Clock Status
; R15-16 Scratch
; R17-R22 Scratch 
; R23 is the Down Counter
; R24 Short Touch Counter
; R25 Idelay and Delay routines (temp storage)
; R26-R31 are X, Y, and Z pointers
; R30 is OutPortB store register
;   X points to Alarm Circuit 1 and 2 commands

;*****Initialization

INITIALIZE1:

ldi	R31,120
mov	R5,R31		; put 120 count in R5 for clock update
ldi	R31,60
mov	R6,R31		; put 60 count into R6 for clock update

sub	R0,R0		;set up count down timer data to zero
sts	$009C,R0
clr	R7		;clear flag register
inc	R7		; turn on accurate time mode for default

ldi	r16,1		;put AM (1) into R10
mov	r10,r16

ldi	r16,8		;put 8 into R9 (hours time (8:00 a.m.))
mov	r9,r16

clr	r16
mov	r8,r16
mov	r6,r16		;set zero in both minutes and seconds counters
mov	r5,r16		;zero 120 counter

			;clock is now reset to 8:00 a.m.

START1:

ser	R30		;Set to all ones for out port direction as output
out	DDRB,R30	;Set PORTD to output
clr	R30
out	PORTB,R30	;LEDs are on when port pins are high

ldi	R28,0		;Set bits to 0 as input for port D
out	DDRD,R28	;execute data direction for port D
out	PORTD,R28	;execute to set ports low, with no pullup resistor

ldi 	r31,low(RAMEND)		; Load low byte address of end of RAM into register R16
out	SPL,R31			; Initialize stack pointer to end of internal RAM

rjmp	EndlessClock

	
RESET1:

					; Set Up Stack Pointer
ldi 	r31,low(RAMEND)		; Load low byte address of end of RAM into register R16
out	SPL,R31			; Initialize stack pointer to end of internal RAM

cbr	R30,1		;turn off Red LED
cbr	R30,2		;turn off Green LED
cbr	R30,4		;turn off Sonalert channel
out	PORTB,R30	


;***** Short switch touches executes commands.  Long Touch and hold down (observing
;        number of red blinks selects programming modes for time and alarms
 
;***** High on port B turns on LEDs
;      Also, other functions for long touch and beginning touches


PowerUp:

rcall	DetectCnt		;wait for key entry

cpi	R23,1			;check downcnt (R23) to see if long touch occured
brsh	LongTouch		;branch is same or higher

cpi	R24,1			;R24 is TPCntr containing number of short touches.
breq	Toggle1a
rjmp	cont50

Toggle1a:

rcall	Toggle1			;toggle 1 circuit
ldi	R31,1			;beep sonalert if active
rcall	BeepSonalert
rjmp	EndlessClock


CONT50:
cpi	R24,2	
breq	Toggle2a
rjmp	Cont51

Toggle2a:

rcall	Toggle2
ldi	R31,2		;beep sonalert 2 times if active
rcall	BeepSonalert
rjmp	EndlessClock		;toggle 2 circuit

Cont51:

cpi	R24,3			;toggle 3 circuit
breq	Toggle3A
rjmp	Cont52

Toggle3a:

rcall	Toggle3
ldi	R31,3		;beep sonalert 3 times if active
rcall	BeepSonalert
rjmp	EndlessClock


Cont52:

cpi	R24,4
breq	Toggle4a
rjmp	Cont53


Toggle4a:

rcall	Toggle4
ldi	R31,4		;beep sonalert 4 times if active
rcall	BeepSonalert
rjmp	EndlessClock


Cont53:

cpi	R24,5			;show alarm1 circuit command memory
breq	ReadOutAlarm1A
rjmp	Cont54

ReadOutAlarm1A:

rcall	ReadOutAlarm1
rjmp	EndlessClock

Cont54:

cpi	R24,6
breq	ReadOutAlarm2A		;show alarm 2 circuit commands
rjmp	Cont55

ReadOutAlarm2A:

rcall	ReadOutAlarm2
rjmp	EndlessClock

Cont55:			;default for every other entry to quarter clock

rjmp	EndlessClock


;  -----------End of short touch selections...


; *********** Start of long touch selections


LongTouch:

cpi	R23,1
breq	SetAlarm1a
rjmp	Cont70		;set A alarm (10 possible entries)

SetAlarm1a:
rcall	SetAlarm1
rjmp	EndlessClock

Cont70:

cpi	R23,2
breq	SetAlarm2a		;set B alarm (10 possible entries)
rjmp	Cont71

SetAlarm2a:

rcall	SetAlarm2
rjmp	EndlessClock

Cont71:

cpi	R23,3
breq	SetTimeA		;set time
rjmp	Cont72

SetTimeA:

rcall	SetTime
rjmp	EndlessClock

Cont72:

cpi	R23,4			
breq	ClearAlarm1a		;clear alarm 1 commands
rjmp	Cont73

ClearAlarm1a:

rcall	ClearAlarm1
rjmp	EndlessClock

Cont73:

cpi	R23,5	
breq	ClearAlarm2a		;clear alarm 2 command
rjmp	Cont74

ClearAlarm2a:

rcall	ClearAlarm2
rjmp	EndlessClock


Cont74:

cpi	R23,6
breq	CntDwnSetTa
rjmp	Cont75

CntDwnSetTa:

rcall	CntDwnSet
rjmp	EndlessClock


Cont75:

cpi	R23,7			;is it 7? Toggle Sonalert alarm flag on/off
breq	SonT1
rjmp	Cont76

SonT1:

rcall	SonalertOnOff
rjmp	EndlessClock


Cont76:

cpi	R23,8		;is it 8?  Toggle clock display accurate/quarter time
breq	ChangeClkA
rjmp	Cont77


ChangeClkA:

rcall	ChangeClockDisplay
rjmp	EndlessClock


Cont77:

rjmp	EndlessClock


; -----------END of long touch commands


; ************************


DetectCnt:

sub	R24,R24		;zero TPCntr
ldi	R31,205		; (50 counts)
mov	R3,R31		;Set up these variables for MaxCnt use
sub	R4,R4
sub	R15,R15		; - max count before timeout

Start2:

sbic	PIND,0x00	;skip if pin 0, port D switch down
rjmp	Start4		;recycle until button is pushed

rcall	DeBounceDly	;allow switch to settle bouncing

sbic	PIND,0x00	;skip if pin 0, port D is still down
rjmp	Start2		;try again to check for button push
rjmp	Cont20

Start4:

MaxCnt:

dec	R3
brne	MaxCntT1

ldi	R31,205		;reset up 205 count (50 counts)
mov	R3,R31
rcall	UpDateClk	;update the clock timers
dec	R4
brne	MaxCntT1

dec	R15
brne	MaxCntT1

rjmp	EndlessClock

MaxCntT1:

rjmp	Start2		;return to check on switch position

Cont20:

DownCount:		; R23 will contain DownCnt 

sub 	R23,R23		;Zero DownCnt variable

DCPT1a:

sub	R11,R11		;zero
sub	R12,R12		;zero
ldi	R22,3		;delay between long touches

DCPT1:

sbic	PIND,0x00	;skip over if pin 0, port D is cleared
rjmp	DCPT2a		;pin 0 must be high

rcall	DeBounceDly	;pause to let bounce settle

DCPT1c:

sbic	PIND,0x00	;skip over in pin 0, port D is cleared
rjmp	DCPT1		

dec	R11		;switch is still down
brne	DCPT1c

push	R22		; save contents of R22 (it will be altered by UpDateClk)
rcall	UpDateClk	;update clock counters
pop	R22		; restire value in R22
dec	R12
brne	DCPT1c

dec	R22
brne	DCPT1c
inc	R23		;increment Down Counter
rcall	FlashRLEDNI	;flash red once with no interrupt from switch

rjmp	DCPT1a


DCPT2a:			;switch is up

rcall	DeBounceDly
sbic	PIND,0x00
rjmp	DCPT3		;switch is up
rjmp	DCPT1		;switch is down - do a recheck

DCPT3:

rcall	DebounceDly

ldi	R31,205		;put 205 count into R11
mov	R11,R31
sub	R12,R12
sub	R13,R13		;R13 checked later determines time switch up interval

inc	R24		;increment Touch Plate Counter - R24


START3:

sbis	PIND,0x00	;skip if switch on PortD is up
rjmp	Start2

UpCntr:
;  UpCounter subroutine - Active countdown delay, incrementing and returning
;  Requires clearing of DecCnt, DecCnt1, DecCnt2 before
;  entering this routine


Up1:

inc 	R11
brne	UpOut

ldi	R31,205		; reset count to 205 count in R11
mov	R11,R31
rcall	UpDateClk

inc	R12
brne	UpOut

inc	R13		;contains count of how long switch has been up (UpCnt)

UpOut:

mov	R31,R13
cpi	R31,10
brlo	Start3		;branch if lower than 10

ret


; ------------------------------------

;To catch non-touch of Touch Plate for X seconds


;MaxCnt is a counter that is used for exiting idle periods without
;  a pushbutton being depressed.  Be sure to preset R3, R4, and R15


UpDateClk:
		;updates clock timers for time displays.  R14 is Clock Bit Status
sbrc	R7,2		;skip if 3rd bit in register 7 is clear
rjmp	UPCT1		;left branch

sbis	PIND,0x01	; check 60 Hz signal line (this is right branch)
rjmp	UPCT5		; to exit

ldi	R31,4		
or	R7,R31		;make R7 non-zero -- set flag
rjmp	UPCT2


UPCT1:
sbic	PIND,0x01	; check 60 Hz signal line (this is left branch)
rjmp	UPCT5		; exit	

ldi	R31,251
and	R7,R31		; set R7, 3rd bit to zero -- clear flag

UPCT2:
dec	R5		; 120 counter for one second interval
brne	UPCT5		; exit

ldi	R31,120		; reload 120 count into R5
mov	R5,R31

dec	R6		; 60 counter for one minute interval
brne	UPCT5

ldi	R31,60
mov	R6,R31		; reload 60 count into R6

inc	R8		;increment minutes count (R8)

rcall	CntDwnDet	;Go check if count down timer set and for match

mov	R16,R8
cpi	R16,60
brsh	UPCT3
rjmp	UPCT4

UPCT5:
ret

UPCT3:
sub	R8,R8		;zero 60 Minute count
inc	R9
mov	R16,R9		;check if count is 12 for AM/PM toggle
cpi	R16,12
breq	UPCT3a
rjmp	UPCT8b		; to continue

UPCT3a:			;toggle am/pm flag

mov	R16,R10
cpi	R16,1		;toggle A.M. / P.M. flag
brne	UPCT8a
ldi	R16,2
mov	R10,R16		;set to PM = 2
rjmp	UPCT8b

UPCT8a:

dec	R10		;Set bit to A.M. = 1


UPCT8b:			;continue

mov	R16,R9
cpi	R16,13		;limit hours to from 1 to 12
brsh	UPCT6
rjmp	UPCT4

UPCT6:
sub	R9,R9
inc	R9		;reset hours to 1


UPCT4:
rcall	AlarmChk
ret

; -----------------------

AlarmChk:

ldi	R26,$60		;start of store locations, Alarm A
clr	R27		;zero High byte

ldi	R22,20		;20 increments for checking Alrm 1 and 2 commands

ACT1:
			;load data pointed at by X pair then inc X
ld	R19,X+		;minutes	
ld	R20,X+		;hours
ld	R21,X+		;AM/PM

rcall	DoChk

dec	R22
brne	ACT1

ret

; ----------------------------

DoChk:		;checks to see if there is a time match
		;R19,20,21 have values of time to be checked

cp	R21,R10	;check AM/PM for match
brne	DCKT5	;return .. no match

cp	R20,R9	;check hours for match
brne	DCKT5	;return .. no match

cp	R19,R8	;check minutes for match
brne	DCKT5	;return .. no match

cpi	R26,$7E		;check for Alarm 1 or 2 command
brlo	Toggle1		;it is Alarm 1


Toggle2:

ldi	R31,16		;Alarm 2 bit selected for toggle
rcall	ToggleABit

DCKT5:

ret


Toggle1:

ldi	R31,8		;Alarm 1 bit selected
rcall	ToggleABit	

ret


Toggle3:

ldi	R31,32		;Alarm 3 bit selected
rcall	ToggleABit	

ret


Toggle4:

ldi	R31,64		;Alarm 4 bit selected
rcall	ToggleABit	

ret
	
;+++++++++++++++++++++++++++++++++++++++++

; This series of delays can NOT be interrupted by switch down

Delay02:		; .2 second delay

push	R0
push	R1
push	R2

sub	R0,R0		;zero registers
sub	R1,R1
ldi	R25,1
mov	R2,R25

Delay02T:		

dec 	R0
brne	Delay02T
rcall	UpDateClk
dec	R1
brne	Delay02T
dec	R2
brne	Delay02T

pop	R2
pop	R1
pop	R0

ret

Delay04:		; .4 second delay

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,2
mov	R2,R25
rjmp	Delay02T	; into Delay02 routine ending


Delay005:		; .05 second delay

push	R0
push	R1

sub	R0,R0
ldi	R25,128
mov	R1,R25

Delay005T:

dec	R0
brne	Delay005T
rcall	UpDateClk	;refresh clock count
dec	R1
brne	Delay005T

pop	R1
pop	R0

ret


Delay1:			; one second delay

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,5
mov	R2,R25
rjmp	Delay02T


Delay2:

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,10
mov	R2,R25
rjmp	Delay02T


DeBounceDly:		;Debounce Delay

push	R0
sub	R0,R0

DeBounceDlyT:

dec	R0
brne	DeBounceDlyT
pop	R0
rcall	UpDateClk	;update clock counters

ret


IDelay02:		; Interruptible .2 S Delay by switch being depressed

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,1
mov	R2,R25

IDelay02T:

dec 	R0
brne	IDelay02T
rcall	UpDateClk
sbis	PIND,0x00	; Skip if button not pushed
rjmp	RESET1		; Goes to RESET1
dec	R1
brne	IDelay02T
dec 	R2
brne	IDelay02T

pop	R2
pop	R1
pop	R0

ret

IDelay04:		; .4 second interruptible delay

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,2
mov	R2,R25
rjmp	Idelay02T	; use ending for IDelay02


IDelay005:		; .05 second interruptible dely

push	R0
push	R1

sub	R0,R0
ldi	R25,128		;load 128 count to R1
mov	R1,R25

IDelay005T:

dec 	R0
brne	IDelay005T
rcall	UpDateClk
sbis	PIND,0x00	; Skip if button not pushed
rjmp	RESET1		; Goes to RESET1
dec	R1
brne	IDelay005T

pop	R1
pop	R0

ret




IDelay1:		; one second interruptible delay

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,5
mov	R2,R25
rjmp	IDelay02T

IDelay2:

push	R0
push	R1
push	R2

sub	R0,R0
sub	R1,R1
ldi	R25,10
mov	R2,R25
rjmp	IDelay02T


; ************** LED Flashers

FlashRed:		;Flash red once

sbr	R30,1
out	PORTB,R30	; on Red LED
rcall	IDelay005
cbr	R30,1
out	PORTB,R30
rcall	IDelay005

ret

FlashGreen:		;Flash green LED once

sbr	R30,2		;green on
out	PORTB,R30
rcall	IDelay005
cbr	R30,2		;off LED
out	PORTB,R30
rcall	IDelay005

ret


FlashRG:		;Flash R and G once
sbr	R30,1
out	PORTB,R30	;on red LED
rcall	IDelay005
cbr	R30,1		;off Red
sbr	R30,2		;on Green
out	PORTB,R30
rcall	IDelay005
cbr	R30,2
out	PORTB,R30	;off green
rcall	IDelay005

ret


VarRFlash:		;R31 contains increments - Variable short flash of red routine

push	R17		;save contents
mov	R17,R31		;put into R17

VRF2:

tst	R17
breq	VRF1
rcall	FlashRed
dec	R17
rjmp	VRF2

VRF1:
pop	R17

ret


VarGFlash:		;R31 contains increments - Variable short flash of green routine

push	R17
mov	R17,R31

VGF2:

tst	R17
breq	VGF1
rcall	FlashGreen
dec	R17
rjmp	VGF2

VGF1:

pop	R17

ret


VarRDFlash:		;R31 = No. increments - Variable long flash of red routine

push	R17
mov	R17,R31

VRDF2:

tst	R17
breq	VRDF1
rcall	FlashRed
rcall	IDelay04
dec	R17
rjmp	VRDF2

VRDF1:

pop	R17

ret


VarGDFlash:		;R31 = No. of increments - Variable long flash of green routine

push	R17
mov	R17,R31

VGDF2:

tst	R17
breq	VGDF1
rcall	FlashGreen
rcall	IDelay04
dec	R17
rjmp	VGDF2

VGDF1:

pop	R17

ret


VarRGFlash:	;R31 contains number of increments to flash red and green

push	R17
mov	R17,R31

VRGF2:

tst	R17
breq	VRGF1
rcall	FlashRG
dec	R17
rjmp	VRGF2


VRGF1:

pop	R17

ret


FlashRLEDNI:		;flash red LED with no switch interrupt possible

sbr	R30,1
out	PORTB,R30
rcall	Delay005
cbr	R30,1
out	PORTB,R30

ret


; ************ 

DisplayClock1:	;R7, bit 0, 1 = accurate, 0 = quarter time display
		;R4=mins, R15=hours, R23= AM/PM ... temp store
		
mov	R4,R8	;move minutes into R4 -- store for processing
mov	R15,R9	;move hours into R15
mov	R23,R10	;move AM/PM into R23 (AM=1, PM=2)

sbrc	R7,0	;skip if bit 0 is cleared -- (go to quarter time)
rjmp	DC1	;jump to accurate time display mode


DC2:		;it is quarter time display mode

ldi	R31,2
rcall	VarRGFlash 	;flashes R/G two times
rjmp	DC3

DC1:		;it is accurate time display mode

ldi	R31,6
rcall	VarRGFlash	;flashes R/G six times


DC3:		;time to display hours

rcall	IDelay1	;one second pause

mov	R31,R15	;load hours for variable flashes

sbrs	R23,0	;skip if it is morning (Green) Display
rjmp	DC3a

rcall	VarGDFlash	;(AM) flash in green, number of hours
rjmp	DC4

DC3a:			;it is PM ... a red flashing

rcall	VarRDFlash	;flash in red, number of hours

DC4:

rcall	IDelay1		;one second pause
sbrs	R7,0		;skip if it is accurate time display
rjmp	DC6

DC5:			;it is accurate time display

sub	R17,R17		;zero R17
mov	R31,R4

DC5e:

inc	R17		;advance MSD by one
subi	R31,10		;subtract 10 from R4 (R31)

brcc	DC5e		;repeat until carry flag gets set

DC5a:			;correct values and continue

ldi	R29,10		
add	R31,R29		;add ten back into value
mov	R4,R31		;store remaining minutes into R4 (LSD)
dec	R17		;correct to right value

sbrs	R23,0		;skip if AM value
rjmp	DC5a2		;go to PM series commands

DC5a1:			;these are AM series of commands

mov	R31,R17
rcall	VarGDFlash	;(AM) flash green MSD number of times
rcall	IDelay1		;one second pause
mov	R31,R4		;move LSD into R17
rcall	VarGDFlash	;(AM) flash green LSD number of time
rjmp	DC7		;to go out

DC5a2:			;these are PM series commands

mov	R31,R17
rcall	VarRDFlash	;(PM) flash red MSD number of times
rcall	IDelay1		;one second pause
mov	R31,R4		;move LSD into R17
rcall	VarRDFlash	;(PM) flash red LSD number of time
rjmp	DC7		;to go out


DC6:			;Display Minutes in Quarter hour times

sub	R15,R15		;zero R15
inc	R15		;put 1 into R15 for flash number start
mov	R31,R4		;save total minutes into R31

cpi	R31,15		; 0-14 minutes
brlo	DC6a

inc	R15		;to two 

cpi	R31,30		; 15-29 minutes
brlo	DC6a

inc 	R15		;to three

cpi	R31,45		; 30-44 minutes
brlo	DC6a

inc	R15		;to four (45-59 minutes)

DC6a:			;R15 value now set

rcall	IDelay1		;one second pause
mov	R31,R15		;put value into R31
sbrs	R23,0		;skip if AM
rjmp	DC6a2		;it is PM

DC6a1:			;it is an AM green flash series

rcall	VarGDFlash
rjmp	DC7		;exit

DC6a2:			;it is a PM red flash series

rcall	VarRDFlash
rjmp	DC7


DC7:		;pause and return

rcall	IDelay1		
rcall	IDelay1		;pause two seconds

ret

		
; ----------------

SetAlarm1:

ldi	R31,5		;flash red 5 times showing start of programming
rcall	VarRFlash
ldi	R26,$60		;start of Alarm 1 memory section
clr	R27		;zero R27


SA10:

rcall	SetAlarmX
cpi	R26,$7E
brlo	SA10		;repeat	
rjmp	SA50


SetAlarm2:

ldi	R31,10		;flash red 10 times to signal start of programming
rcall	VarRFlash
ldi	R26,$7E		;start of Alarm 2 memory section
clr	R27		;zero R27

SA11:

rcall	SetAlarmX	
cpi	R26,$9C
brlo	SA11


SA50:

rcall	FlashRed
rcall	Delay02
rcall	FlashRG
rcall	FlashRed

rjmp	EndlessClock	;go to clock display


SetAlarmX:	;routine used for both Set Alarm 1 and 2

push	R26		; save contents of X register pair
push	R27

rcall	DetectCnt	;uses LONG holdown of button for count entry
cpi	R23,0		;check for zero entry to exit routine (short holddown)
breq	SA50		;exit

cpi	R23,3		;check for too high a number entry
brlo	SA1b		;to continue process
rjmp	SA50		;exit -- wrong entry


SA1b:

mov	R2,R23		;save AM/PM value of 1=AM, 2=PM
rcall	FlashRed	;flash red once indicating first entry okay

SA1:

rcall	DetectCnt
cpi	R23,13		;check for right value entry
brlo	SA1c		;to continue process

rjmp	SA50		; exit


SA1c:			

mov	R1,R23		;store away hours
rcall	FlashRed	;flash red indicating entry okay

SA2:

rcall	DetectCnt	
cpi	R23,7		;check if number greater than six
brlo	SA2a		;continue process
rjmp	SA50		; exit


SA2a:

clr	R0		;clear R0 - minute counter
ldi	R31,10		;put ten in R31

SA2a1:

tst	R23
breq	SA2a2		;check for zero
add	R0,R31		;add ten value to R3
dec	R23		;one less for R23
rjmp	SA2a1		;keep adding ten

SA2a2:			;R3 contains MSD * 10 now

rcall	FlashRed	; flash red indicating entry okay


SA3:

rcall	DetectCnt
cpi	R23,10
brlo	SA4
rjmp	SA50		; exit -- wrong entry


SA4:

add	R0,R23		;R3 now has full minute count
rcall	FlashRed
rcall	Delay02
rcall	FlashRed	;double flash -- indicating value series entered okay.

pop	R27		; restore x register pair
pop	R26		

st	X+,R0		;store away minutes
st	X+,R1		;store away hours
st	X+,R2		;store away AM/PM

ret

; ---------------------

ReadOutAlarm1:

ldi	R31,5		; 5 iterations
rcall	VarGFlash	;signal entering ReadOutAlarm1 section
rcall	IDelay04	;wait
ldi	R26,$60		;start of Alarm 1 data commands
clr	R27		;clear X register hi byte

RO1:

rcall	ReadOutX
cpi	R26,$7E		;end of memory for alarm 1
brlo	RO1


RO50:

rcall	Delay04
rcall	Delay04
ldi	R31,3
rcall	VarRFlash	; 3 red flashes indicate leaving ReadOutAlarm section


rjmp	EndlessClock


ReadOutAlarm2:

ldi	R31,15		;15 iterations signals entering ReadoutAlarm2 section
rcall	VarGFlash
rcall	IDelay04	;wait
ldi	R26,$7E		;start of Alarm 2 data commands
clr	R27		;clear X register hi byte

RO2:

rcall	ReadOutX
cpi	R26,$9C		;end of memory for alarm 2
brlo	RO2		;continue

rjmp	RO50		;exit routine


ReadOutX:		;common to ReadOutAlarm1 and 2

ld	R0,X+		;mins
ld	R1,X+		;hours
ld	R2,X+		;AM/PM

push 	R26		; save contents of X register pair
push	R27

mov	R31,R2
tst	R31		;check for zero (no command)
breq	RoX49		;exit

mov	R31,R2
cpi	R31,3		;check for larger value than 2
brlo	RoX1		;to continue

rjmp	RoX49		;exit (invalid number)


RoX1:

rcall	IDelay1		;wait one second

mov	R31,R2		
rcall	VarRDFlash	;Flash 1=AM 2=PM
rcall	IDelay1

mov	R31,R1		;put hours in
rcall	VarRDFlash	;Flash red number of hours
rcall	IDelay1

sub	R2,R2		;zero R2 for MSD minutes count

ldi	R31,10
mov	R1,R31		;put ten count in R1 for later processing
mov	R31,R0		;put R0 (minutes) into R31 for processing

RoX1a:

inc	R2		;add a count of MSD
sub	R31,R1		;subtract ten minutes
brcc	RoX1a		;repeat as necessary

dec	R2		;restore correct value
add	R31,R1		;restore correct LSD minutes value in R31
mov	R1,R31		;put LSD value into R1

mov	R31,R2		;MSD value of minutes
rcall	VarRDFlash	;flash MSD times -- red
rcall	IDelay1		;wait one second

mov	R31,R1		;load in LSD value of minutes
rcall	VarRDFlash
rcall	IDelay1		;wait one second


RoX49:

pop	R27		; restore contents of X register pair
pop	R26

ret

; ------------------------------


SetTime:

ldi	R31,60		;set 60 into counter for one minute interval
mov	R6,R31		;put in register - (prevents count from jumping during time set)

sub	R29,R29		;zero minutes accumulator
ldi	R31,10		;ten flashes of green
rcall	VarGFlash

rcall	DetectCnt	
cpi	R23,3		;check AM/PM validity
brlo	SetTimeT1
ret			; invalid entry

SetTimeT1:

cpi	R23,0
breq	SetTimeT4	;return - invalid entry

mov	R10,R23		;put AM/PM value in place
rcall	Delay02
rcall	FlashRed	;verify input

rcall	DetectCnt
mov	R9,R23		;put in HOURS value
rcall	Delay02
rcall	FlashRed	;verify input

rcall	DetectCnt

SetTimeT2:

tst	R23		;check value
breq	SetTimeT3	;branch if zero
ldi	R17,10
add	R29,R17		;add 10 to R29 (ten minute increments)
dec	R23
rjmp	SetTimeT2

SetTimeT3:

rcall	Delay02
rcall	FlashRed	;verify MSD of minutes entered

rcall	DetectCnt
add	R29,R23		;add in LSD to total
mov	R8,R29		;store complete minutes value

ldi	R31,60		;set 60 into counter for one minute interval
mov	R6,R31		;put in register - zeros seconds to 00

rcall	Delay02
rcall	FlashRed	;verify entry


SetTimeT4:

ret

;-----------------------------


EndlessClock:		;displays accurate time

rcall	DisplayClock1
rjmp	EndlessClock	;loop forever


; --------------

ClearAlarm1:

sub	R28,R28		;zero R28
ldi	R31,30		;30 increments
sub	R27,R27		;MSB for X register
ldi	R26,$60		;start of Alarm 1 data

CAT1:

tst	R31
breq	CAT2
st	X+,R28
dec	R31
brne	CAT1

CAT2:

ret


ClearAlarm2:

sub	R28,R28		;zero R28
ldi	R31,30		;30 increments
sub	R27,R27		;high byte zero
ldi	R26,$7E		;start of Alarm 2 data

CAT2a:

tst	R31
breq	CAT2b
st	X+,R28		;zero into location, then increment
dec	R31
brne	CAT2a

CAT2b:

ret

; -------------------

ToggleABit:		;Toggles selected bits in Output PortB

push	R0
			;R31 contains mask such as 1,2,4, etc.
mov	R0,R30		;save portB bits
com	R0		;complement all bits
and	R0,R31		;R31 mask selects changed bits and store them in R15
com	R31		;change mask
and	R30,R31		;zero bits to be changed by mask
or	R30,R0		;insert changed bits
out	PortB,R30	;execute

pop	R0

ret

; -------------------------


Test1:		;R31 contains value to flash RED LED

push	R31
push	R26
push 	R17

mov	R26,R31		;save R31 number of increments
rcall	IDelay2

Test1a:

tst	R26
breq	testend

ldi	R31,128		;toggle bit 128
rcall	ToggleABit
rcall	IDelay02
ldi	R31,128
rcall	ToggleABit
rcall	IDelay04


dec	R26
rjmp	Test1a

testend:

rcall	IDelay2
pop	R17
pop	R26
pop	R31

ret

; ----------------------------

CntDwnSet:		;Sets delay time for count down timer

ldi	R31,2		;Flash green two times
rcall	VarGFlash


CntDwnSet1:		;(short touch is a zero entry)

rcall	DetectCnt	;Wait for entry of long touches - hours (4 max)
cpi	R23,5		;jump out if entry is too high
brsh	CntDwnSet49	; error -jump out

rcall	FlashRed	;confirm entry

ldi	R31,60
mov	R0,R31		;put 60 into R0
mov	R31,R23
sub	R23,R23		;zero R23 and convert to temp accumulator

CntDwnSet2:

tst	R31
breq	CntDwnSet3

add	R23,R0		;adds 60 (one hour) to R23 (accumulator)
dec	R31
rjmp	CntDwnSet2

CntDwnSet3:

mov	R1,R23		;save minutes total in R1


CntDwnSet4:

rcall	DetectCnt	;get MSD minutes

cpi	R23,6
brsh	CntDwnSet49	;jump out if => 6

rcall	FlashRed	;confirm entry and delay

ldi	R31,10
mov	R0,R31		;put ten into R0
mov	R31,R23
sub	R23,R23		;save R23 value into R31 then zero R23

CntDwnSet4a:

tst	R31
breq	CntDwnSet5

add	R23,R0
dec	R31
rjmp	CntDwnSet4a


CntDwnSet5:

add	R1,R23		;put new total in R1 accumulator

CntDwnSet6:

rcall	DetectCnt
cpi	R23,10
brsh	CntDwnSet49	; jump out if entry is 10 or bigger

rcall	FlashRed	;confirm entry

add	R1,R23		;Total now complete

sts	$009C,R1	;store direct to data space 9C

ldi	R31,32		;to toggle circuit 3
rcall	ToggleABit
rjmp	CntDwnSet50	; to continue

CntDwnSet49:

sub	R1,R1		; zero R1 and store zero - incorrect entry made
sts	$009C,R1

CntDwnSet50:

ret

; -----------------------------------

CntDwnDet:	;to detect a zero of the count down timer and toggle circuit 3
		; otherwise return
		
lds	R31,$009C	;get down count from $9C
tst	R31
breq	CntDwnDet50	;(return)

dec	R31
brne	CntDwnDet2
 
ldi	R31,32		;toggle circuit 3 bit
rcall	ToggleABit
sub	R31,R31		;zero count down


CntDwnDet2:

sts	$009C,R31	;store away decremented value

CntDwnDet50:

ret

; --------------------------------------

SonalertOnOff:		;toggles Sonalert flag on or off

rcall	FlashGreen	;signal entering Sonalert on/off set
rcall	Delay1		;one second delay

mov	R31,R7		;move into R31 for alteration
sbrc	R31,1		;skip if bit 1 is clear
rjmp	SOO1		;sonalert is on ...  go turn it off

sbr	R31,2		;set bit 1 in register R31 (turn it on)
mov	R7,R31		;move r31 to r7 with changed bit

sbr	R30,4		; beep to indicate change made to on
out	PortB,R30
rcall	Delay04	
cbr	R30,4		; Off sonalert i/o
out	PortB,R30

ret

SOO1:			; Sonalert flag is on .... turn it off.

cbr	R31,2		;clear bit 1 in register
mov	R7,R31		;move change back into R7

ret

; ------------------------------

BeepSonalert:

sbrc	R7,1		;skip if bit 1 clear
rjmp	BSon1		; if flag set, beep sonalert R31 times

ret

BSon1:

tst	R31
breq	Bson2

dec	R31
push 	R31		; preserve R31 No. of Times value
sbr	R30,4
out	PortB,R30
rcall	Delay04
cbr	R30,4
out	PortB,R30
rcall	Delay04
pop	R31		; restore R31
rjmp	BSon1

BSon2:

ret

; -----------------------------


ChangeClockDisplay:	; toggles clock display between accurate (1) and quarter (0) time


mov	R31,R7		;move R7 contents into R31 for changes
sbrs	R31,0		;skip if bit 0 in register 31 is set
rjmp	CCDisplay1

cbr	R31,1		;clear bit 0 in register 31
mov	R7,R31		;move change back into R3

ret


CCDisplay1:

sbr	R31,1		;set bit 0 in register 31
mov	R7,R31		;move change back into R3

ret

;-----------------------------------------